# -*- coding: utf-8 -*-
# @Author: wangkun
# @Time: 2022/4/18

"""
下载并上传：播放量视频
"""
import os
import sys

sys.path.append(os.getcwd())
from main.common import Common
from main.get_feeds import get_feeds
from main.publish import Publish
from main.feishu_lib import Feishu


class DownloadPlay:

    @staticmethod
    def play_rule(play_width, play_height, play_duration, play_play_cnt):
        """
        1.分辨率，宽或者高 >= 720 or ==0
        2.600s >= 时长 >= 60s
        3.视频播放量 >= 150000
        """
        if int(play_width) >= 720 or int(play_height) >= 720 or play_width == "0" or play_height == "0":
            if 600 >= int(play_duration) >= 60:
                if int(play_play_cnt) >= 150000:
                    return True
                else:
                    return False
            else:
                return False
        else:
            return False

    @classmethod
    def download_play_video(cls, env):
        """
        下载播放量视频
        测试环境:env == dev
        正式环境:env == prod
        """
        if len(Feishu.get_values_batch("SdCHOM")) == 1:
            pass
        else:
            for i in range(len(Feishu.get_values_batch("SdCHOM"))):
                try:
                    download_video_id = Feishu.get_values_batch("SdCHOM")[i+1][1]
                    download_video_play_cnt = Feishu.get_values_batch("SdCHOM")[i+1][2]
                    download_video_title = Feishu.get_values_batch("SdCHOM")[i+1][3]
                    download_video_duration = Feishu.get_values_batch("SdCHOM")[i+1][4]
                    download_video_comment_cnt = Feishu.get_values_batch("SdCHOM")[i+1][5]
                    download_video_like_cnt = Feishu.get_values_batch("SdCHOM")[i+1][6]
                    download_video_share_cnt = Feishu.get_values_batch("SdCHOM")[i+1][7]
                    download_video_resolution = Feishu.get_values_batch("SdCHOM")[i+1][8]
                    download_video_width = download_video_resolution.split("*")[0]
                    download_video_height = download_video_resolution.split("*")[-1]
                    download_video_send_time = Feishu.get_values_batch("SdCHOM")[i+1][9]
                    download_user_name = Feishu.get_values_batch("SdCHOM")[i+1][10]
                    download_head_url = Feishu.get_values_batch("SdCHOM")[i+1][11]
                    download_cover_url = Feishu.get_values_batch("SdCHOM")[i+1][12]
                    download_video_url = Feishu.get_values_batch("SdCHOM")[i+1][13]
                    download_video_session = Feishu.get_values_batch("SdCHOM")[i+1][14]

                    if cls.play_rule(download_video_width, download_video_height,
                                     download_video_duration, download_video_play_cnt) is True:
                        Common.logger().info("开始下载视频:{}", download_video_title)

                        # 下载封面
                        Common.download_method(text="cover", d_name=download_video_title, d_url=download_cover_url)
                        # 下载视频
                        Common.download_method(text="video", d_name=download_video_title, d_url=download_video_url)
                        # 保存视频信息至 "./videos/{download_video_title}/info.txt"
                        with open(r"./videos/" + download_video_title + r"/info.txt", "a", encoding="utf8") as f_a:
                            f_a.write(str(download_video_id) + "\n" +
                                      str(download_video_title) + "\n" +
                                      str(download_video_duration) + "\n" +
                                      str(download_video_play_cnt) + "\n" +
                                      str(download_video_comment_cnt) + "\n" +
                                      str(download_video_like_cnt) + "\n" +
                                      str(download_video_share_cnt) + "\n" +
                                      str(download_video_resolution) + "\n" +
                                      str(download_video_send_time) + "\n" +
                                      str(download_user_name) + "\n" +
                                      str(download_head_url) + "\n" +
                                      str(download_video_url) + "\n" +
                                      str(download_cover_url) + "\n" +
                                      str(download_video_session))

                        # 上传视频
                        Common.logger().info("开始上传视频:{}", download_video_title)
                        Publish.upload_and_publish(env, "play")

                        # 保存视频 ID 到云文档:https://w42nne6hzg.feishu.cn/sheets/shtcngRPoDYAi24x52j2nDuHMih?sheet=20ce0c
                        Common.logger().info("保存视频ID至云文档:{}", download_video_title)
                        # 看一看+ ，视频ID工作表，插入首行
                        Feishu.insert_columns("20ce0c")
                        # 看一看+ ，视频ID工作表，首行写入数据
                        Feishu.update_values("20ce0c", download_video_id, "", "", "",
                                             "", "", "", "", "", "", "", "", "", "", "")

                        # 从云文档删除该视频信息:https://w42nne6hzg.feishu.cn/sheets/shtcngRPoDYAi24x52j2nDuHMih?sheet=SdCHOM
                        Common.logger().info("从云文档删除该视频信息:{}", download_video_title)
                        # 删除行或列，可选 ROWS、COLUMNS
                        Feishu.dimension_range("SdCHOM", "ROWS", i + 2, i + 2)

                    else:
                        # 从云文档删除该视频信息:https://w42nne6hzg.feishu.cn/sheets/shtcngRPoDYAi24x52j2nDuHMih?sheet=SdCHOM
                        Common.logger().info("该视频不满足下载规则，删除在云文档中的信息:{}", download_video_title)
                        # 删除行或列，可选 ROWS、COLUMNS
                        Feishu.dimension_range("SdCHOM", "ROWS", i + 2, i + 2)

                except Exception as e:
                    Common.logger().error("视频 info 异常，删除该视频信息", e)
                    # 删除行或列，可选 ROWS、COLUMNS
                    Feishu.dimension_range("SdCHOM", "ROWS", i + 2, i + 2)

            cls.download_play_video("prod")


if __name__ == "__main__":
    download_play = DownloadPlay()
    get_feeds()
    download_play.download_play_video("dev")
